#include "TROOT.h"
#include "TEnv.h"
#include "TMath.h"
#include "TObjArray.h"
#include "TObjString.h"
#include "TList.h"
#include "TSystem.h"
#include "TString.h"
#include "TTimeStamp.h"
#include <fstream>
#include <iostream>
#include <fstream>

namespace std {} using namespace std;
//_____________________________________________________________________________
const char tex_prolog[] =
"\\documentclass[a4paper,12pt]{article}\n\
\\usepackage{graphicx}\n\
\\usepackage{thumbpdf}\n\
\\usepackage[polutonikogreek,german]{babel}\n\
\\usepackage[utf8x]{inputenx}\n\
\n\
\\newcommand{\\greek}[1]{{\\selectlanguage{polutonikogreek}#1}}\n\
\n\
%% \\usepackage[a4paper,\n\
%% pdfdisplaydoctitle=true,\n\
%% pdfmenubar=false,\n\
%% pdftoolbar=false,\n\
%% pdfview=FitV\n\
%% ]{hyperref}\n\
\n\
%% \\usepackage{mathptmx}\n\
\\usepackage[scaled=.90]{helvet}\n\
\\usepackage{courier}\n\
\n\
\\usepackage{geometry}\n\
\\geometry{a4paper,\n\
     left=7mm,right=7mm,\n\
     top=1mm,bottom=1mm}\n\
\n\
\\usepackage{color}\n\
\\definecolor{red}{rgb}{1,0.,0.}\n\
\\definecolor{grey}{rgb}{0.4,0.4,0.4}\n\
\\definecolor{greyred}{rgb}{0.6,0.,0.}\n\
%% \\renewcommand {\\L}{\\Large}\n\
%% \\renewcommand {\\l}{\\large}\n\
\\renewcommand {\\c}{\\color}\n\
\\setlength{\\parindent}{0pt}\n\
\\begin{document}\n\
\n\
\\pagestyle{empty}\n\
";

//________________________________________________________________________

int GetDaysOfMonth(int year, int m);
int GetDayOfEaster(int y); 
int GetLastSunday(int year, int month);
int GetDay(int year, int month, int  week, int  dofweek, int &weekinyear);

//_____________________________________________________________________________

	const char * month_name_de [] = 
	{"Januar", "Februar", "M\\\"arz", "April", "Mai", "Juni",
	"Juli", "August", "September", "Oktober", "November", "Dezember"};
	const char * month_name_en [] = 
	{"January", "February", "March", "April", "May", "June",
	"July", "August", "September", "October", "November", "December"};
	const char * month_name_fr [] = 
	{"Janvier", "Fevrier", "Mars", "Avril", "May", "Juin",
	"Julliet", "Aout", "Septembre", "Octobre", "Novembre", "Decembre"};
	const char * month_name_es [] = 
	{"Enero", "Febrero", "Marzo", "Abril", "Mayo", "Junio",
	"Julio", "Augusto", "Septiembre", "Octubre", "Noviembre", "Deciembre"};
	const char * month_name_it [] = 
	{"Gennaio", "Febbraio", "Marzo", "Aprile", "Maggio", "Giugno",
	"Luglio", "Agosto", "Settembre", "Ottobre", "Novembre", "Dicembre"};
	
	const char * weekday_name_de [] = 
	{"Montag\\hfill", "Dienstag\\hfill", "Mittwoch\\hfill",
	"Donnerstag\\hfill", "Freitag\\hfill", "Samstag\\hfill", "Sonntag\\hfill"};
	const char * weekday_name_en [] = 
	{"Monday\\hfill", "Tuesday\\hfill", "Wednesday\\hfill",
	"Thursday\\hfill", "Friday\\hfill", "Saturday\\hfill", "Sunday\\hfill"};
	const char * weekday_name_fr [] = 
	{"Lundi\\hfill", "Mardi\\hfill", "Mercredi\\hfill",
	"Jeudi\\hfill", "Vendredi\\hfill", "Samedi\\hfill", "Dimanche\\hfill"};
	const char * weekday_name_es [] = 
	{"Lunes\\hfill", "Martes\\hfill", "M\\\'iercoles\\hfill",
	"Jueves\\hfill", "Viernes\\hfill", "S\\\'abado\\hfill", "Domingo\\hfill"};
	const char * weekday_name_it [] = 
	{"Lunedi\\hfill", "Martedi\\hfill", "Mercoledi\\hfill",
	"Giovedi\\hfill", "Venerdi\\hfill", "Sabato\\hfill", "Domenica\\hfill"};
	
	const char * kw_de ="KW";
	const char * kw_en ="cw";
	const char * kw_fr ="Se";
	const char * kw_it ="Se";
	const char * kw_es ="Se";
	
//________________________________________________________________________

class SpDay : public TObject
{
private:
	TString fName;
	int   fDate;
	int   fIsHoliday;
public:
	SpDay(const char *name, int date, int isholiday);
	~SpDay() {};
	TString * GetHolName() {return &fName;};
	int   GetDate() {return fDate;};
	int   IsHoliday() {return fIsHoliday;};
	Bool_t DateMatches(int date);
ClassDef (SpDay, 0)
};

SpDay::SpDay(const char *name, int date, int isholiday)
{
	fName = name;
	fDate = date;
	fIsHoliday = isholiday;
}
//_______________________________________________________________________

Bool_t SpDay::DateMatches(int date)
{
// if fDate does not contain year date is valid for all years (e.g. christmas date)	
	int t = date;
	if (fDate < 10000) 
		t %= 10000;
	if ( t == fDate )
		return kTRUE;
	else
		return kFALSE;
}
//_______________________________________________________________________

class Holidays : public TObject
{
private:
	const char * newy;        
	const char * epiphany; 
	const char * labor_day;    
	const char * assumption;
	const char * nat_holiday;
	const char * all_hallows;  
	const char * christmas_eve;       
	const char * christmas_sun;    
	const char * christmas_mon;   
	const char * newyears_eve;     

	const char * good_friday;   
	const char * easter_sun;       
	const char * easter_mon; 
	const char * ascension;  
	const char * pentecost_sun;   
	const char * pentecost_mon;
	const char * corpus_christi;
	const char * summertime;
	const char * wintertime;
	
	int fLabor_day_date;
	int fNat_holiday_date;
	int fYear;
	int fWriteVorl;
	TString fLang;
	TString fSpecialDaysFile;
	TList fList;
	public:
		Holidays(int year, const char * lang = "de",int gWriteVorl=1 );
		~Holidays() {};
		void Init(int year);
		void AddDay(const char * day, int date, int isholiday);
		TList * GetList() { return &fList; };
		int GetYear() { return fYear;};
		int GetLabor_day_date() { return fLabor_day_date;};
		int GetNat_holiday_date(){ return fNat_holiday_date;};
ClassDef (Holidays, 0)
};
//_______________________________________________________________________

Holidays::Holidays(int year, const char * lang, int gWriteVorl)
{
	fLang = lang;
	fYear = year;
	fWriteVorl = gWriteVorl;
	Init(year);
}
//_______________________________________________________________________

void Holidays::AddDay(const char * day, int date, int isholiday)
{
	fList.Add(new SpDay(day, date, isholiday));
}
//_______________________________________________________________________

void Holidays::Init(int year)
{
// 	cout << "Holidays::Init(int year) " << year<< endl;
	fYear = year;
	const char * newy_de           = "Neujahr";        
	const char * epiphany_de     	 = "3 K\\\"onige"; 
	const char * labor_day_de      = "Tag der Arbeit";    
	const char * assumption_de     = "Maria HF";
	const char * nat_holiday_de    = "Tag der Einheit";
	const char * all_hallows_de    = "Allerheiligen";  
	const char * christmas_eve_de  = "Hl Abend";       
	const char * christmas_sun_de  = "Weihnachten";    
	const char * christmas_mon_de  = "Weihnachten";   
	const char * newyears_eve_de   = "Sylvester";     

	const char * good_friday_de    = "Karfreitag";   
	const char * easter_sun_de     = "Ostern";       
	const char * easter_mon_de     = "Ostermontag"; 
	const char * ascension_de      = "Himmelfahrt";  
	const char * pentecost_sun_de  = "Pfingsten" ;   
	const char * pentecost_mon_de  = "Pfingstmontag";
	const char * corpus_christi_de = "Fronleichnam" ;
	const char * summertime_de     = "Sommerzeit";
	const char * wintertime_de     = "Winterzeit";
	
	const char * newy_en           = "New Year";        
	const char * epiphany_en       = "Epiphany"; 
	const char * labor_day_en      = "Labor Day";    
	const char * assumption_en     = "Assumption";
//	const char * nat_holiday_en    = "Nat Holiday";
	const char * all_hallows_en    = "All Hallows";  
	const char * christmas_eve_en  = "Christmas Eve";       
	const char * christmas_sun_en  = "Christmas";    
	const char * christmas_mon_en  = "Christmas";   
	const char * newyears_eve_en   = "New Years Eve";     

	const char * good_friday_en    = "Good Friday";   
	const char * easter_sun_en     = "Easter";       
	const char * easter_mon_en     = "Easter"; 
	const char * ascension_en      = "Ascension";  
	const char * pentecost_sun_en  = "Pentecost" ;   
	const char * pentecost_mon_en  = "Pentecost";
	const char * corpus_christi_en = "Corpus Christi" ;
	const char * summertime_en     = "Sommertime";
	const char * wintertime_en     = "Wintertime";
	
	const char * newy_fr           = "Nouvel An";        
	const char * epiphany_fr       = "Rois"; 
	const char * labor_day_fr      = "Fete du Travail";    
	const char * assumption_fr     = "Assomption";
	const char * nat_holiday_fr    = "14 Juillet";
	const char * all_hallows_fr    = "Toussaint";  
	const char * christmas_eve_fr  = "R\\`eveillon";       
	const char * christmas_sun_fr  = "No\\\"el";    
	const char * christmas_mon_fr  = "";   
	const char * newyears_eve_fr   = "R\\`eveillon";     

	const char * good_friday_fr    = "";   
	const char * easter_sun_fr     = "P\\^aques";       
	const char * easter_mon_fr     = "P\\^aques"; 
	const char * ascension_fr      = "Ascension";  
	const char * pentecost_sun_fr  = "Pentec\\^ote" ;   
	const char * pentecost_mon_fr  = "Pentec\\^ote";
	const char * corpus_christi_fr = "" ;
	const char * summertime_fr     = "Heure d'\\\'et\\\'e";
	const char * wintertime_fr     = "Heure d'hiver";
	
	const char * newy_it           = "Capodanno ";        
	const char * epiphany_it       = "Epifania"; 
	const char * labor_day_it      = "Festa del Lavoro";    
	const char * assumption_it     = "Assunzione";
	const char * nat_holiday_it    = "Festa della Repubblica";
	const char * all_hallows_it    = "Tutti i santi";  
	const char * christmas_eve_it  = "";       
	const char * christmas_sun_it  = "Natale";    
	const char * christmas_mon_it  = "Santo Stefano";   
	const char * newyears_eve_it   = "San Silvestro";     

	const char * good_friday_it    = "";   
	const char * easter_sun_it     = "P\\^aques";       
	const char * easter_mon_it     = "Luned\\`i \\\\dell 'Angelo"; 
	const char * ascension_it      = "Ascensione";  
	const char * pentecost_sun_it  = "Pentec\\^ote" ;   
	const char * pentecost_mon_it  = "Pentec\\^ote";
	const char * corpus_christi_it = "" ;
	const char * summertime_it     = "Stagione estiva";
	const char * wintertime_it     = "Stagione invernale";
	
	const char * newy_es           = "A\\~no Nuevo";        
	const char * epiphany_es       = "Reyes Magos"; 
	const char * labor_day_es      = "D\\`ia internacional de los Trabajadores";    
	const char * assumption_es     = "Asunci\\'on";
	const char * nat_holiday_es    = "Dia de la Hispanidad";
	const char * all_hallows_es    = "Todos los Santos";  
	const char * christmas_eve_es  = "Nochebuena";       
	const char * christmas_sun_es  = "Navidad";    
	const char * christmas_mon_es  = "Navidad";   
	const char * newyears_eve_es   = "Nochevieja";     

	const char * good_friday_es    = "Viernes Santo";   
	const char * easter_sun_es     = "Pascua";       
	const char * easter_mon_es     = "Pascua"; 
	const char * ascension_es      = "Ascensi\\\'on";  
	const char * pentecost_sun_es  = "Pentecost\\\'es" ;   
	const char * pentecost_mon_es  = "Pentecost\\\'es";
	const char * corpus_christi_es = "Corpus Christi" ;
	const char * summertime_es     = "Hora de Verano";
	const char * wintertime_es     = "Hora de Invierno";
	
	if        (fLang == "en" || fLang == "us" ) {
		newy           = newy_en          ;
		epiphany       = epiphany_en      ;
		labor_day      = labor_day_en     ;
		assumption     = assumption_en    ;
		all_hallows    = all_hallows_en   ;
		christmas_eve  = christmas_eve_en ;      
		christmas_sun  = christmas_sun_en ;  
		christmas_mon  = christmas_mon_en ;  
		newyears_eve   = newyears_eve_en  ;   

		good_friday    = good_friday_en   ;
		easter_sun     = easter_sun_en    ;   
		easter_mon     = easter_mon_en    ;
		ascension      = ascension_en     ;
		pentecost_sun  = pentecost_sun_en ;  
		pentecost_mon  = pentecost_mon_en ;
		corpus_christi = corpus_christi_en;
		summertime     = summertime_en    ;
		wintertime     = wintertime_en    ;
		if (fLang == "en" ) {
			fLabor_day_date   = 501;
			fNat_holiday_date = -1;
			nat_holiday    = ""  ;
		} else {
			fLabor_day_date   = 0;
			fNat_holiday_date = 704;
			nat_holiday    = "Independence Day"  ;
		}
	} else if (fLang == "fr" ) {
		newy           = newy_fr          ;
		epiphany       = epiphany_fr    ;
		labor_day      = labor_day_fr     ;
		assumption     = assumption_fr    ;
		nat_holiday    = nat_holiday_fr   ;
		all_hallows    = all_hallows_fr   ;
		christmas_eve  = christmas_eve_fr ;      
		christmas_sun  = christmas_sun_fr ;  
		christmas_mon  = christmas_mon_fr ;  
		newyears_eve   = newyears_eve_fr  ;   

		good_friday    = good_friday_fr   ;
		easter_sun     = easter_sun_fr    ;   
		easter_mon     = easter_mon_fr    ;
		ascension      = ascension_fr     ;
		pentecost_sun  = pentecost_sun_fr ;  
		pentecost_mon  = pentecost_mon_fr ;
		corpus_christi = corpus_christi_fr;
		summertime     = summertime_fr    ;
		wintertime     = wintertime_fr    ;
		fLabor_day_date   = 501;
		fNat_holiday_date = 714;
	} else if (fLang == "es" ) {
		newy           = newy_es          ;
		epiphany       = epiphany_es      ;
		labor_day      = labor_day_es     ;
		assumption     = assumption_es    ;
		nat_holiday    = nat_holiday_es   ;
		all_hallows    = all_hallows_es   ;
		christmas_eve  = christmas_eve_es ;      
		christmas_sun  = christmas_sun_es ;  
		christmas_mon  = christmas_mon_es ;  
		newyears_eve   = newyears_eve_es  ;   

		good_friday    = good_friday_es   ;
		easter_sun     = easter_sun_es    ;   
		easter_mon     = easter_mon_es    ;
		ascension      = ascension_es     ;
		pentecost_sun  = pentecost_sun_es ;  
		pentecost_mon  = pentecost_mon_es ;
		corpus_christi = corpus_christi_es;
		summertime     = summertime_es    ;
		wintertime     = wintertime_es    ;
		fLabor_day_date   = 501;
		fNat_holiday_date = 1012;
		
	} else if (fLang == "it" ) {
		newy           = newy_it          ;
		epiphany       = epiphany_it      ;
		labor_day      = labor_day_it     ;
		assumption     = assumption_it    ;
		nat_holiday    = nat_holiday_it   ;
		all_hallows    = all_hallows_it   ;
		christmas_eve  = christmas_eve_it ;      
		christmas_sun  = christmas_sun_it ;  
		christmas_mon  = christmas_mon_it ;  
		newyears_eve   = newyears_eve_it  ;   

		good_friday    = good_friday_it   ;
		easter_sun     = easter_sun_it    ;   
		easter_mon     = easter_mon_it    ;
		ascension      = ascension_it     ;
		pentecost_sun  = pentecost_sun_it ;  
		pentecost_mon  = pentecost_mon_it ;
		corpus_christi = corpus_christi_it;
		summertime     = summertime_it    ;
		wintertime     = wintertime_it    ;
		fLabor_day_date   = 501;
		fNat_holiday_date = 602;
	} else {
		newy           = newy_de          ;
		epiphany       = epiphany_de      ;
		labor_day      = labor_day_de     ;
		assumption     = assumption_de    ;
		nat_holiday    = nat_holiday_de   ;
		all_hallows    = all_hallows_de   ;
		christmas_eve  = christmas_eve_de ;      
		christmas_sun  = christmas_sun_de ;  
		christmas_mon  = christmas_mon_de ;  
		newyears_eve   = newyears_eve_de  ;   

		good_friday    = good_friday_de   ;
		easter_sun     = easter_sun_de    ;   
		easter_mon     = easter_mon_de    ;
		ascension      = ascension_de     ;
		pentecost_sun  = pentecost_sun_de ;  
		pentecost_mon  = pentecost_mon_de ;
		corpus_christi = corpus_christi_de;
		summertime     = summertime_de    ;
		wintertime     = wintertime_de    ;
		fLabor_day_date   = 501;
		fNat_holiday_date = 1003;
	}
	fList.Delete();
	
	// Vorlesungszeiten
	if ( fLang == "de" && fWriteVorl == 1 ) {
		fList.Add(new SpDay("Vorlesungsende",  20120211, 0));
		fList.Add(new SpDay("Vorlesungsbeginn",20120416, 0));
		fList.Add(new SpDay("Vorlesungsende",  20120721, 0));
		fList.Add(new SpDay("Vorlesungsbeginn",20121015, 0));
		fList.Add(new SpDay("Vorlesungsende",  20130209, 0));
		fList.Add(new SpDay("Vorlesungsbeginn",20130415, 0));
		fList.Add(new SpDay("Vorlesungsende",  20130720, 0));
		fList.Add(new SpDay("Vorlesungsbeginn",20131014, 0));
		fList.Add(new SpDay("Vorlesungsende",  20140208, 0));
		fList.Add(new SpDay("Vorlesungsbeginn",20140407, 0));
		fList.Add(new SpDay("Vorlesungsende",  20140712, 0));
		fList.Add(new SpDay("Vorlesungsbeginn",20141006, 0));
		fList.Add(new SpDay("Vorlesungsende",  20150131, 0));
	}
	// add fixed holidays
	if (fLang == "fr") {
		fList.Add(new SpDay("Armistice 1918",  1109 , 1));
		fList.Add(new SpDay("Victoire 1945",    508 , 1));
	}
	if (fLang == "it") {
		fList.Add(new SpDay("Festa della Liberazione",  425 , 1));
		fList.Add(new SpDay("Immacolata concezione",   1208 , 1));
	}
	fList.Add(new SpDay(newy          ,  101 , 1));
	fList.Add(new SpDay(epiphany      ,  106 , 1));
	fList.Add(new SpDay(labor_day     ,  fLabor_day_date , 1));
	fList.Add(new SpDay(assumption    ,  815 , 1));
	fList.Add(new SpDay(nat_holiday   ,  fNat_holiday_date , 1));
	fList.Add(new SpDay(all_hallows   ,  1101 , 1));
	fList.Add(new SpDay(christmas_eve ,  1224 , 1));
	fList.Add(new SpDay(christmas_sun ,  1225 , 1));
	if (strlen(christmas_mon) > 1)
		fList.Add(new SpDay(christmas_mon , 1226 , 1));
	fList.Add(new SpDay(newyears_eve  ,   1231 , 1));
//	cout << " Add " << newyears_eve << " "<< year*10000+1231 << endl;
	
	// Wintertime starts on last Sunday of October 
	int wd = GetLastSunday(year, 10);
	fList.Add(new SpDay(wintertime ,  year*10000+1000 + wd , 0));
	wd = GetLastSunday(year, 3);
	fList.Add(new SpDay(summertime  ,  year*10000+300 + wd , 0));
		
	// moving holidays depend on easter
	// sunday after full moon after spring equinox
	// Ostersonntag, can by April or March
	int de = GetDayOfEaster(year);
	int me = 4;
	if ( de < 0 ) {
		de = -de;
		me = 3;
	}
	TTimeStamp easter(year, me, de, 0,0,0);
	TTimeStamp temp;
	
	// Karfreitag
	int mkf = me;
	int kf = de - 2;
	if (kf <= 0) {
		kf += 30;
		mkf = 3;
	}
	if (strlen(good_friday) > 1)
		fList.Add(new SpDay(good_friday,  year*10000+100*mkf+kf , 1));
	fList.Add(new SpDay(easter_sun,  year*10000+100*me+de , 1));
	// Ostermontag
	// nb: UNIX time 0 is 1. Jan 1970
	TTimeStamp off1(1970, 1,2, 0,0,0);
	temp = easter;
	temp.Add(off1);
	fList.Add(new SpDay(easter_mon ,  temp.GetDate() , 1));
	
	TTimeStamp off40(1970, 2,9, 0,0,0);
	temp = easter;
	temp.Add(off40);
	fList.Add(new SpDay(ascension ,   temp.GetDate() , 1));
	
	TTimeStamp off50(1970, 2,19, 0,0,0);
	temp = easter;
	temp.Add(off50);
	fList.Add(new SpDay(pentecost_sun ,  temp.GetDate() , 1));
	
	TTimeStamp off51(1970, 2,20, 0,0,0);
	temp = easter;
	temp.Add(off51);
	fList.Add(new SpDay(pentecost_mon ,  temp.GetDate() , 1));
	
	TTimeStamp off61(1970, 2,30, 0,0,0);
	temp = easter;
	temp.Add(off61);
	fList.Add(new SpDay(corpus_christi ,  temp.GetDate() , 1));
	
	// add days defined by user
	
	TEnv env("makeCal.conf");
	TString sdate;
	for (int m=1; m <= 12; m++) {
		for (int d=1; d<= GetDaysOfMonth(year, m); d++){
			// only month and day given?
			sdate = "";
			sdate += (m*100+d);
			if (sdate.Length() == 3)
				sdate.Prepend("0");
			if (env.Lookup(sdate) ){
				fList.Add(new SpDay(env.GetValue(sdate,"nn") , year*10000+ m*100+d, 0));
				//cout << sdate << " " << env.GetValue(sdate,"nn") << endl;
			}
			// is year given?
			sdate = "";
			sdate += (year*10000+m*100+d);
			if (env.Lookup(sdate) ){
				fList.Add(new SpDay(env.GetValue(sdate,"nn") , year*10000+ m*100+d, 0));
				//cout << sdate << " " << env.GetValue(sdate,"nn") << endl;
			}
			
		}
	}
}
//_______________________________________________________________________

void makeCal(int month_in = 0, int year_in = 0, int nof_month_in = 0, const char * lang = "")
{
	if (month_in <= 0 ) {
		cout << endl <<
" This program generates LaTeX code for a calendar sheet of one month" << endl <<
" optionally together with a picture on top" << endl <<
" Without a picture upto 3 months / page are possible" << endl <<
" Calling sequence: " << endl <<
" \"makeCal(month, year, gNofMonths, lang)\"" << endl <<
" Only month is mandatory, if year is ommitted the current is taken" << endl <<
" The following languages are supported: \"de\",\"en\",\"es\",\"fr\",\"it\",\"us\","<< endl <<
" Holidays are correct for the years 1900-2099 and approx. complete for "<< endl <<
" \"de\" and \"fr\" only" << endl <<
endl <<
" If a picture on top is requested the program assumes a directory \"Pictures\""<< endl <<
"(name may be changed in \"makeCal.conf\") with the following files:"<< endl <<
" \"01.jpg\" etc, one picture for each month (01 : January)"<< endl <<
" and \"01.txt\" etc,  the picture captions"<< endl <<
" In fact pictures can have any format accepted by \"pdflatex\":" << endl <<
" jpg, gif, png..  however not eps (so 01.jpg may be a symlink)"<< endl << endl <<
" The program \"knows\" start of summer / wintertime  and" << endl << 
" Vorlesungszeiten of the LMU." << endl << endl<<
" Some features may be configured in \"makeCal.conf\":" << endl <<
" Consult also the provided \"makeCal.conf\" for more comments" << endl <<
" WriteYear: 1" << endl <<
" WriteWeek: 0" << endl <<
" WriteVorl: 1" << endl <<
" NofMonths: 3" << endl <<
" MonthsPerPage: 3" << endl <<
" DrawBoxes:  0" << endl <<
" FontFamily: ptm" << endl <<
" NumberSize: \\large" << endl <<
" Lang:       de" << endl <<
" WithPicture: 0" << endl <<
" PictureDir: Pictures" << endl <<
" 1206: Nikolaus" << endl <<
" 20120612: \\bf HEP Conference"<< endl <<
endl <<
" User defined entries are also read from \"makeCal.conf\":" << endl <<
" \"YYYYMMDD: text\" like \"20120512: HEP Conference\"" << endl <<
" If date is independent of year it should be omitted like: 1206: Nikolaus" << endl
		<< endl;
		return;
	};
	
	const char * p_name [] = {	"01.jpg", "02.jpg", "03.jpg", "04.jpg", "05.jpg", "06.jpg",
										"07.jpg", "08.jpg", "09.jpg", "10.jpg", "11.jpg", "12.jpg"};

	const char * c_name [] = {	"01.txt", "02.txt", "03.txt", "04.txt", "05.txt", "06.txt",
										"07.txt", "08.txt", "09.txt", "10.txt", "11.txt", "12.txt"};
	
	int month = month_in;
	int year = year_in;
	TString gLang(lang);
	TTimeStamp ts;
	if (year == 0)
		year = ts.GetDate()/10000;  // current year
	int gNofMonths = nof_month_in;
	TEnv env("makeCal.conf");
   TString number_size = env.GetValue("NumberSize", "\\large");
//	number_size.Prepend("\\");
	int gWriteYear = env.GetValue("WriteYear", 1);
	int gWriteWeek = env.GetValue("WriteWeek", 1);
	int gWriteVorl = env.GetValue("WriteVorl", 1);
	int gDrawBoxes = env.GetValue("DrawBoxes", 0);
	
	if (gNofMonths == 0) 
		gNofMonths = env.GetValue("NofMonths", 3);
	
	int gWithPicture = env.GetValue("WithPicture", 1);
	TString picture_name, picture_dir;
	if ( gWithPicture == 1 ) {
		picture_dir = env.GetValue("PictureDir", "Pictures");
		picture_dir.Append("/");
		picture_name = picture_dir;
		picture_name +=  p_name[month-1];
		Long_t id, size, flags, modtime;
		if (gSystem->GetPathInfo(picture_name.Data(), &id, &size, &flags, &modtime)) {
			cout << "No picture found" << endl;
			gWithPicture  = 0;
		} 
	}
	Double_t fheight = 1.;
	int gMonthsPerPage = env.GetValue("MonthsPerPage", 3);
	if (gWithPicture == 1) {
		gNofMonths = 1;
		gMonthsPerPage = 1;
		
	} else {
		fheight = 3. / gMonthsPerPage;
	}
	Double_t fwidth = 2.5;
	if (gWriteWeek == 1) 
		fwidth = 2.3;
	Double_t extraspace = 0;
	if (gMonthsPerPage < 3)
		extraspace = 0.1;
	TString gFontFamily = env.GetValue("FontFamily", "ptm");
	if (gLang.Length() < 2)
		gLang= env.GetValue("Lang", "de");
	
	const char * month_name [12];
	for (int m = 0; m < 12; m++) {
		if        (gLang == "en" || gLang == "us") {
			month_name[m] = month_name_en[m];
		} else if (gLang == "fr") {
			month_name[m] = month_name_fr[m];
		} else if (gLang == "es") {
			month_name[m] = month_name_es[m];
		} else if (gLang == "it") {
			month_name[m] = month_name_it[m];
		} else {
			month_name[m] = month_name_de[m];
		}
	}
	const char * weekday_name[7];
	
	for (int m = 0; m < 7; m++) {
		if        (gLang == "en" || gLang == "us") {
			weekday_name[m] = weekday_name_en[m];
		} else if (gLang == "fr") {
			weekday_name[m] = weekday_name_fr[m];
		} else if (gLang == "es") {
			weekday_name[m] = weekday_name_es[m];
		} else if (gLang == "it") {
			weekday_name[m] = weekday_name_it[m];
		} else {
			weekday_name[m] = weekday_name_de[m];
		}
	}
	
	const char * kw;
	if        (gLang == "en" || gLang == "us") {
		kw = kw_en;
	} else if (gLang == "fr") {
		kw = kw_fr;
	} else if (gLang == "es") {
		kw = kw_es;
	} else if (gLang == "it") {
		kw = kw_it;
	} else {
		kw = kw_de;
	}

	
	int mf = month + gNofMonths -1;
	if ( mf > 12 )
		mf -= 12;
	TString tex_out("m");
	tex_out += Form("%02d", month_in);
	if (gNofMonths > 1) {
		tex_out += "_";
		tex_out += Form("%02d", mf);
	}
	tex_out += ".tex";
	
	Holidays * hol = new Holidays(year, gLang.Data(), gWriteVorl);
	
	ofstream outf;
	outf.open(tex_out, ios::out);
	
	outf << tex_prolog;
	outf << "\\newlength{\\fheight}" << endl;
	outf << "\\newlength{\\fwidth}" << endl;
	outf << "\\setlength{\\fheight}{" << fheight << "cm}" << endl;
	outf << "\\setlength{\\fwidth}{"  << fwidth  << "cm}" << endl;
	outf << "\\newlength{\\extraspace}" << endl;
	outf << "\\setlength{\\extraspace}{"  << extraspace  << "cm}" << endl;	
	outf << "\\fontfamily{"<< gFontFamily << "}\\selectfont" << endl;
	outf << "\\renewcommand{\\bf}{\\bfseries}" << endl;
	if  ( gWithPicture == 1 ) {
		outf << "\\vspace*{1cm}" << std::endl;
		outf << "\\begin{center}" << std::endl;
		outf << "\\parbox{17.5cm}{" << std::endl;
		outf << "    \\includegraphics[ width=17.5cm,clip=true,keepaspectratio=true,trim=10 0 10 0]{";

		outf <<  picture_name << "}" << std::endl;
		outf << "}\\\\[.25cm]" << std::endl;
		TString caption;
		TString caption_name(picture_dir);
		caption_name += c_name[month-1];
		ifstream cfile(caption_name.Data());
		if ( cfile.is_open() ) {
			caption.ReadLine(cfile);
			outf  << "{\\bf " << caption << "}" <<  std::endl;
			cfile.close();
		}
		outf  << std::endl << "\\vspace*{1cm}" << std::endl;
		outf << "\\end{center}" << std::endl;
	}
	
	// loop on months
	for (int mo = 1; mo <= gNofMonths; mo++) {
		outf  << std::endl << "\\vspace*{\\extraspace}" << std::endl;
		outf << "\\begin{center}" << std::endl;
		outf << "{\\Large\\bf " ;
		outf << month_name[month-1];
		if (gWriteYear > 0) {
			outf << " " <<  year;
		}
		outf << "\\\\[\\extraspace] }" << std::endl;
		if (gWriteWeek) {
			outf << "\\begin{tabular*}{\\textwidth}{c @{\\extracolsep{\\fill}}c@{}c@{}c@{}c@{}c@{}c@{}c@{}}" << std::endl;
			outf << "\\rule{0.0cm}{0.7cm} \\bf\\scriptsize " << kw << "&";
		} else {
			outf << "\\begin{tabular*}{\\textwidth}{@{\\extracolsep{\\fill}}c@{}c@{}c@{}c@{}c@{}c@{}c@{}}" << std::endl;
		}
		for (int i = 0; i < 7; i++) {
			outf << "\\rule{0.0cm}{0.7cm} \\bf " << weekday_name[i];
			if (i < 6)
				outf << " & " ;
		}
		outf << "\\\\[\\extraspace] " << std::endl;
	//	outf << "\\hline" << std::endl;  //boxes
		const char * hname = NULL;
		int isholiday;
		int winy;
		for (int w = 1; w <= 6; w++) {
			for (int d = 1; d <= 7; d++) {
				int day = GetDay(year, month, w, d, winy);
				int pday = day;
				// if more than 1 month req and not last month only write
				// 5 weeks if month fits
				if ( pday < 0 && w > 1 && mo%gMonthsPerPage)
					break;
				int pmonth = month;
				int pyear = year;
				// optionally put week number 
				if ( gWriteWeek && d == 1 ) {
					outf << "\\begin{minipage}[t][\\fheight][t]{0.4cm}\\begin{center}\\scriptsize\\c{blue}"<< winy << "\\end{center}\\end{minipage} &";
				}

				// days not belonging to this month are negative
				if ( pday < 0 ) {
					pday = -pday;
					if ( w == 1 ) {
						pmonth -= 1;
						if ( pmonth <= 0 ) {
							pmonth = 12;
							pyear -= 1;
						}
					} else {
						pmonth += 1;
						if ( pmonth > 12 ) {
							pmonth = 1;
							pyear += 1;
						}
					}
						
				} 
				// if day belongs to last / next year we need to recalculate holidays
				if ( pyear != hol->GetYear() )
					hol->Init(pyear);
				
				// care about holiday and user defined days
				isholiday = 0;	
				TString hday;
				int cur_date = 10000*pyear + 100*pmonth + pday;
				TIter next(hol->GetList());
				SpDay * spday; 
				while ( (spday = (SpDay*)next()) ) {
					if ( spday->DateMatches(cur_date) ) {
//						hday += " ";
//						hday += "\\ ";
						if ( spday->IsHoliday() ) {
							isholiday = 1;
							if (day < 0 )
								hday+="\\c{greyred} ";
							else
								hday+="\\c{red} ";
						}
						hday += *(spday->GetHolName());
					}
				}
//				TString cs("\\c{black} ");
				TString cs("");
				if ( day < 0 ) {
					if (d == 7 || hday.Length() > 0) {
						cs = "\\c{greyred} " ;
					} else {
						cs = "\\c{grey} " ;
					}
				} else {
					if (d == 7 || ( hday.Length()&& isholiday) ) {
						cs = "\\c{red} " ;
					}
				}
				// put day and possibly name of holiday in a minipage
				TString cmd;
				if (gDrawBoxes )
					cmd += "\\fbox{";
				cmd += "\\begin{minipage}[t][\\fheight][t]{\\fwidth}\\begin{center}";
				cmd +=  number_size;
				cmd += cs;
				cmd += pday;
				cmd += "\\end{center}";
				if (hday.Length() > 0) {
					cmd += "\\vspace*{-1.2em}";
					if (!hday.Contains("tiny") && !hday.Contains("size")&& 
						!hday.Contains("small",  TString::kIgnoreCase) )
						cmd += "\\scriptsize ";
					cout << "hday " <<hday << endl;
					cmd += hday;
				}
				cmd += "\\end{minipage} ";
				if ( gDrawBoxes )
					cmd += "}";
				
				outf << cmd;
				if (d < 7) {
					outf << " & " ;
				}
				hname = NULL;
			}
			if ( w < 6) {
				outf << " \\\\[-.2cm]" << std::endl;
			} else if (mo%gMonthsPerPage && gDrawBoxes == 0 ) {
				outf << " \\\\ \\hline" << std::endl;
			}
		}
		outf << "\\end{tabular*}" << std::endl;
	//	outf << "\\end{large}" << std::endl;
		outf << "\\end{center}" << std::endl;
//		if ( mo < gNofMonths-1)
//			outf << "\\vspace*{0.3cm} \\hline" << std::endl;    // for boxes
		if ( gNofMonths > 1 ) {
			month += 1;
			if ( month > 12 ) {
				month = 1;
				year += 1;
			}
		}
		if ( mo%gMonthsPerPage == 0 )
			outf << "\\newpage" << std::endl;
	}
	if ( gNofMonths%gMonthsPerPage != 0 )
		outf << "\\newpage" << std::endl;
	outf << "\\end{document}" << std::endl;	
	cout  <<"LaTeX code written to: " << tex_out << endl;
	return;
}

//_______________________________________________________________________

int GetDaysOfMonth(int year, int m)
{
	if (m == 1 || m == 3 || m == 5 || m == 7 || m == 8|| m == 10 || m == 12 ) {
		return 31;
	} else if (m == 2) {
		if ( TTimeStamp::IsLeapYear(year) ) {
			return 29;
		} else {
			return 28;
		}
	} else {
		return 30;
	}
}
//_______________________________________________________________________

int GetDayOfEaster(int y) 
{
	if (y < 1900 || y > 2099 ) {
	cout<< "Year must be 1900 - 2099: "  << y << endl;	
		return -100;
	}
	int a = (19 * (y % 19) + 24) % 30;
	int d = 22 + a + ((2 * (y % 4) + 4 * (y % 7) + 6 * a + 5) % 7);

	int m = 3;
	if ( d > 31 ) {
		m = 4;
		d-= 31;
	}

	if ( y == 1954 || y == 1981 || y == 2049 || y == 2076 ) {
		d-= 7;
	}

//	cout<< "Easter: "  << y << ": " <<  m << " " << d << endl;
	if (m == 3 )
		return -d;
	else
		return d;
}  
//_______________________________________________________________________

int GetLastSunday(int year, int month)
{
	TTimeStamp ts;
	// get day of 1.
	for (int d = GetDaysOfMonth(year, month); d >= 1; d--) {
		ts.Set(year, month, d, 12,0,0,0,0,0);
		if (ts.GetDayOfWeek() == 7)
			return d;
	}
	return -1;
}	
//_______________________________________________________________________

int GetDay(int year, int month, int  week, int  dofweek, int &weekinyear)
{
	TTimeStamp ts;
	int pmonth = month;
	int pyear = year;
	// get day of 1.
	ts.Set(year, month, 1, 12,0,0,0,0,0);
	int dof1st = ts.GetDayOfWeek();
	
	int day = dofweek + (week - 1) * 7 - dof1st + 1;
	if ( day <= 0 ) {
		pmonth = month -1;
		if (pmonth <= 0) {
			pmonth = 12;
			pyear -= 1;
		}
		
		day = GetDaysOfMonth(year, pmonth) + day;
//		cout << "day " << day << endl;
		day = -day;
	} else if (day > GetDaysOfMonth(year, month)) {
		day = day - GetDaysOfMonth(year, month);
		day = -day;
		pmonth = month +1;
		if (pmonth > 12) {
			pmonth = 1;
			pyear += 1;
		}
	}
	ts.Set(pyear, pmonth, TMath::Abs(day), 12,0,0,0,0,0);
	weekinyear = ts.GetWeek() % 100;
	return day;
}
